from .chat import Chat, AsyncChat
from .embeddings import Embeddings, AsyncEmbeddings
from .dialects import Dialects, AsyncDialects
from .documents import Documents, AsyncDocuments
from .audio import Audio, AsyncAudio
from .models import Models, AsyncModels

__all__ = [
    "Chat",
    "AsyncChat",
    "Embeddings",
    "AsyncEmbeddings",
    "Dialects",
    "AsyncDialects",
    "Documents",
    "AsyncDocuments",
    "Audio",
    "AsyncAudio",
    "Models",
    "AsyncModels",
]



from __future__ import annotations

from typing import Any

from .._types import DialectDetectResponse


class Dialects:
    """Sync dialects: client.dialects.detect(...)"""

    def __init__(self, client: object) -> None:
        self._client = client

    def detect(self, *, text: str, **kwargs: Any) -> DialectDetectResponse:
        payload: dict[str, Any] = {"text": text, **kwargs}

        resp = self._client._http.post(  # type: ignore[attr-defined]
            self._client._build_url("/v1/dialects/detect"),  # type: ignore[attr-defined]
            json=payload,
            headers=self._client._headers,  # type: ignore[attr-defined]
            timeout=self._client.timeout,  # type: ignore[attr-defined]
        )
        self._client._raise_for_status(resp)  # type: ignore[attr-defined]
        return DialectDetectResponse(**resp.json())


class AsyncDialects:
    """Async dialects: client.dialects.detect(...)"""

    def __init__(self, client: object) -> None:
        self._client = client

    async def detect(self, *, text: str, **kwargs: Any) -> DialectDetectResponse:
        payload: dict[str, Any] = {"text": text, **kwargs}

        resp = await self._client._http.post(  # type: ignore[attr-defined]
            self._client._build_url("/v1/dialects/detect"),  # type: ignore[attr-defined]
            json=payload,
            headers=self._client._headers,  # type: ignore[attr-defined]
            timeout=self._client.timeout,  # type: ignore[attr-defined]
        )
        self._client._raise_for_status(resp)  # type: ignore[attr-defined]
        return DialectDetectResponse(**resp.json())



from __future__ import annotations

import httpx

from ._base_client import BaseClient
from .resources.chat import Chat
from .resources.embeddings import Embeddings
from .resources.dialects import Dialects
from .resources.documents import Documents
from .resources.audio import Audio
from .resources.models import Models


class RightNow(BaseClient):
    """Synchronous RightNow API client.

    Usage::

        from rightnow import RightNow

        client = RightNow(api_key="rn-xxx")
        response = client.chat.completions.create(
            model="falcon-h1-arabic-7b",
            messages=[{"role": "user", "content": "مرحبا"}],
        )
        print(response.choices[0].message.content)
    """

    chat: Chat
    embeddings: Embeddings
    dialects: Dialects
    documents: Documents
    audio: Audio
    models: Models

    def __init__(
        self,
        *,
        api_key: str | None = None,
        base_url: str | None = None,
        timeout: float | None = None,
        max_retries: int | None = None,
    ) -> None:
        super().__init__(
            api_key=api_key,
            base_url=base_url,
            timeout=timeout,
            max_retries=max_retries,
        )
        self._http = httpx.Client(timeout=self.timeout, follow_redirects=True)

        self.chat = Chat(self)
        self.embeddings = Embeddings(self)
        self.dialects = Dialects(self)
        self.documents = Documents(self)
        self.audio = Audio(self)
        self.models = Models(self)

    def close(self) -> None:
        self._http.close()

    def __enter__(self) -> RightNow:
        return self

    def __exit__(self, *args: object) -> None:
        self.close()



from __future__ import annotations

from typing import Any, Literal

from pydantic import BaseModel, Field


# ── Chat Completions ────────────────────────────────────────────────────────


class FunctionCall(BaseModel):
    name: str
    arguments: str


class ToolCall(BaseModel):
    id: str
    type: str = "function"
    function: FunctionCall


class ChatMessage(BaseModel):
    role: str
    content: str | None = None
    tool_calls: list[ToolCall] | None = None


class Choice(BaseModel):
    index: int
    message: ChatMessage
    finish_reason: str | None = None


class DeltaContent(BaseModel):
    role: str | None = None
    content: str | None = None
    tool_calls: list[ToolCall] | None = None


class StreamChoice(BaseModel):
    index: int
    delta: DeltaContent
    finish_reason: str | None = None


class UsageInfo(BaseModel):
    prompt_tokens: int = 0
    completion_tokens: int = 0
    total_tokens: int = 0


class ChatCompletion(BaseModel):
    id: str
    object: str = "chat.completion"
    created: int
    model: str
    choices: list[Choice]
    usage: UsageInfo | None = None


class ChatCompletionChunk(BaseModel):
    id: str
    object: str = "chat.completion.chunk"
    created: int
    model: str
    choices: list[StreamChoice]


# ── Embeddings ──────────────────────────────────────────────────────────────


class EmbeddingData(BaseModel):
    object: str = "embedding"
    index: int
    embedding: list[float]


class EmbeddingResponse(BaseModel):
    object: str = "list"
    data: list[EmbeddingData]
    model: str
    usage: UsageInfo | None = None


# ── Models ──────────────────────────────────────────────────────────────────


class ModelInfo(BaseModel):
    id: str
    object: str = "model"
    created: int
    owned_by: str


class ModelListResponse(BaseModel):
    object: str = "list"
    data: list[ModelInfo]


# ── Dialects ────────────────────────────────────────────────────────────────


class DialectScore(BaseModel):
    dialect: str
    confidence: float


class DialectDetectResponse(BaseModel):
    primary_dialect: str
    confidence: float
    scores: list[DialectScore]


# ── Documents ───────────────────────────────────────────────────────────────


class DocumentResponse(BaseModel):
    result: Any
    model: str
    usage: UsageInfo | None = None


# ── Audio ───────────────────────────────────────────────────────────────────


class TranscriptionResponse(BaseModel):
    text: str
    language: str | None = None



from __future__ import annotations

import json

import httpx
import pytest
import respx

from rightnow import RightNow, AsyncRightNow
from rightnow._exceptions import AuthenticationError, BadRequestError, RightNowError


BASE = "https://arabic-api-backend.rightnowai.co"


# ── Init tests ──────────────────────────────────────────────────────────────


def test_requires_api_key():
    import os
    os.environ.pop("RIGHTNOW_API_KEY", None)
    with pytest.raises(RightNowError, match="API key is required"):
        RightNow()


def test_custom_base_url():
    client = RightNow(api_key="test", base_url="http://localhost:8080")
    assert client.base_url == "http://localhost:8080"
    client.close()


# ── Sync chat ───────────────────────────────────────────────────────────────


@respx.mock
def test_chat_completion():
    mock_response = {
        "id": "chatcmpl-123",
        "object": "chat.completion",
        "created": 1700000000,
        "model": "falcon-h1-arabic-7b",
        "choices": [
            {
                "index": 0,
                "message": {"role": "assistant", "content": "أهلاً!"},
                "finish_reason": "stop",
            }
        ],
        "usage": {"prompt_tokens": 10, "completion_tokens": 5, "total_tokens": 15},
    }

    respx.post(f"{BASE}/v1/chat/completions").mock(
        return_value=httpx.Response(200, json=mock_response)
    )

    client = RightNow(api_key="test-key", base_url=BASE)
    result = client.chat.completions.create(
        model="falcon-h1-arabic-7b",
        messages=[{"role": "user", "content": "مرحبا"}],
    )

    assert result.choices[0].message.content == "أهلاً!"
    assert result.usage.total_tokens == 15
    client.close()


@respx.mock
def test_chat_401():
    respx.post(f"{BASE}/v1/chat/completions").mock(
        return_value=httpx.Response(
            401, json={"error": {"message": "Invalid API key"}}
        )
    )

    client = RightNow(api_key="bad-key", base_url=BASE)
    with pytest.raises(AuthenticationError, match="Invalid API key"):
        client.chat.completions.create(
            model="falcon-h1-arabic-7b",
            messages=[{"role": "user", "content": "test"}],
        )
    client.close()


# ── Sync models ─────────────────────────────────────────────────────────────


@respx.mock
def test_list_models():
    mock_response = {
        "object": "list",
        "data": [
            {"id": "falcon-h1-arabic-7b", "object": "model", "created": 1700000000, "owned_by": "rightnow"},
        ],
    }

    respx.get(f"{BASE}/v1/models").mock(
        return_value=httpx.Response(200, json=mock_response)
    )

    client = RightNow(api_key="test-key", base_url=BASE)
    result = client.models.list()
    assert len(result.data) == 1
    assert result.data[0].id == "falcon-h1-arabic-7b"
    client.close()


# ── Sync embeddings ─────────────────────────────────────────────────────────


@respx.mock
def test_embeddings():
    mock_response = {
        "object": "list",
        "data": [{"object": "embedding", "index": 0, "embedding": [0.1, 0.2, 0.3]}],
        "model": "bge-m3-arabic",
        "usage": {"prompt_tokens": 5, "completion_tokens": 0, "total_tokens": 5},
    }

    respx.post(f"{BASE}/v1/embeddings").mock(
        return_value=httpx.Response(200, json=mock_response)
    )

    client = RightNow(api_key="test-key", base_url=BASE)
    result = client.embeddings.create(model="bge-m3-arabic", input="مرحبا")
    assert len(result.data) == 1
    assert result.data[0].embedding == [0.1, 0.2, 0.3]
    client.close()


# ── Sync dialects ───────────────────────────────────────────────────────────


@respx.mock
def test_dialect_detect():
    mock_response = {
        "primary_dialect": "egyptian",
        "confidence": 0.95,
        "scores": [
            {"dialect": "egyptian", "confidence": 0.95},
            {"dialect": "msa", "confidence": 0.03},
        ],
    }

    respx.post(f"{BASE}/v1/dialects/detect").mock(
        return_value=httpx.Response(200, json=mock_response)
    )

    client = RightNow(api_key="test-key", base_url=BASE)
    result = client.dialects.detect(text="ازيك يا حبيبي")
    assert result.primary_dialect == "egyptian"
    assert result.confidence == 0.95
    client.close()


# ── Async chat ──────────────────────────────────────────────────────────────


@respx.mock
@pytest.mark.asyncio
async def test_async_chat_completion():
    mock_response = {
        "id": "chatcmpl-456",
        "object": "chat.completion",
        "created": 1700000000,
        "model": "falcon-h1-arabic-7b",
        "choices": [
            {
                "index": 0,
                "message": {"role": "assistant", "content": "مرحباً بك!"},
                "finish_reason": "stop",
            }
        ],
        "usage": {"prompt_tokens": 10, "completion_tokens": 5, "total_tokens": 15},
    }

    respx.post(f"{BASE}/v1/chat/completions").mock(
        return_value=httpx.Response(200, json=mock_response)
    )

    async with AsyncRightNow(api_key="test-key", base_url=BASE) as client:
        result = await client.chat.completions.create(
            model="falcon-h1-arabic-7b",
            messages=[{"role": "user", "content": "مرحبا"}],
        )
        assert result.choices[0].message.content == "مرحباً بك!"



from __future__ import annotations

from typing import Any

from .._types import ModelInfo, ModelListResponse


class Models:
    """Sync models: client.models.list() / client.models.retrieve(...)"""

    def __init__(self, client: object) -> None:
        self._client = client

    def list(self) -> ModelListResponse:
        resp = self._client._http.get(  # type: ignore[attr-defined]
            self._client._build_url("/v1/models"),  # type: ignore[attr-defined]
            headers=self._client._headers,  # type: ignore[attr-defined]
            timeout=self._client.timeout,  # type: ignore[attr-defined]
        )
        self._client._raise_for_status(resp)  # type: ignore[attr-defined]
        return ModelListResponse(**resp.json())

    def retrieve(self, model_id: str) -> ModelInfo:
        resp = self._client._http.get(  # type: ignore[attr-defined]
            self._client._build_url(f"/v1/models/{model_id}"),  # type: ignore[attr-defined]
            headers=self._client._headers,  # type: ignore[attr-defined]
            timeout=self._client.timeout,  # type: ignore[attr-defined]
        )
        self._client._raise_for_status(resp)  # type: ignore[attr-defined]
        return ModelInfo(**resp.json())


class AsyncModels:
    """Async models: client.models.list() / client.models.retrieve(...)"""

    def __init__(self, client: object) -> None:
        self._client = client

    async def list(self) -> ModelListResponse:
        resp = await self._client._http.get(  # type: ignore[attr-defined]
            self._client._build_url("/v1/models"),  # type: ignore[attr-defined]
            headers=self._client._headers,  # type: ignore[attr-defined]
            timeout=self._client.timeout,  # type: ignore[attr-defined]
        )
        self._client._raise_for_status(resp)  # type: ignore[attr-defined]
        return ModelListResponse(**resp.json())

    async def retrieve(self, model_id: str) -> ModelInfo:
        resp = await self._client._http.get(  # type: ignore[attr-defined]
            self._client._build_url(f"/v1/models/{model_id}"),  # type: ignore[attr-defined]
            headers=self._client._headers,  # type: ignore[attr-defined]
            timeout=self._client.timeout,  # type: ignore[attr-defined]
        )
        self._client._raise_for_status(resp)  # type: ignore[attr-defined]
        return ModelInfo(**resp.json())



from __future__ import annotations

import os
from typing import Any

import httpx

from ._config import DEFAULT_BASE_URL, DEFAULT_MAX_RETRIES, DEFAULT_TIMEOUT, VERSION
from ._exceptions import (
    STATUS_MAP,
    APIConnectionError,
    APITimeoutError,
    InternalServerError,
    RightNowError,
)


class BaseClient:
    """Shared configuration and helpers for sync/async clients."""

    base_url: str
    api_key: str
    timeout: float
    max_retries: int

    def __init__(
        self,
        *,
        api_key: str | None = None,
        base_url: str | None = None,
        timeout: float | None = None,
        max_retries: int | None = None,
    ) -> None:
        self.api_key = api_key or os.environ.get("RIGHTNOW_API_KEY", "")
        if not self.api_key:
            raise RightNowError(
                "API key is required. Pass api_key= or set RIGHTNOW_API_KEY env var."
            )
        self.base_url = (base_url or os.environ.get("RIGHTNOW_BASE_URL", DEFAULT_BASE_URL)).rstrip(
            "/"
        )
        self.timeout = timeout if timeout is not None else DEFAULT_TIMEOUT
        self.max_retries = max_retries if max_retries is not None else DEFAULT_MAX_RETRIES

    @property
    def _headers(self) -> dict[str, str]:
        return {
            "Authorization": f"Bearer {self.api_key}",
            "Content-Type": "application/json",
            "User-Agent": f"rightnow-python/{VERSION}",
        }

    def _build_url(self, path: str) -> str:
        return f"{self.base_url}{path}"

    @staticmethod
    def _raise_for_status(response: httpx.Response) -> None:
        if response.is_success:
            return

        try:
            body = response.json()
        except Exception:
            body = {"error": {"message": response.text}}

        error_msg = body.get("error", {}).get("message", response.text) if isinstance(body, dict) else str(body)
        exc_cls = STATUS_MAP.get(response.status_code, InternalServerError)
        raise exc_cls(error_msg, status_code=response.status_code, body=body)

    @staticmethod
    def _wrap_connection_error(exc: Exception) -> RightNowError:
        if isinstance(exc, httpx.TimeoutException):
            return APITimeoutError(f"Request timed out: {exc}")
        return APIConnectionError(f"Connection error: {exc}")



from __future__ import annotations

from typing import Any, Iterator, AsyncIterator, Union, overload

import httpx

from .._types import ChatCompletion, ChatCompletionChunk
from .._streaming import Stream, AsyncStream


class Completions:
    """Sync chat completions: client.chat.completions.create(...)"""

    def __init__(self, client: object) -> None:
        self._client = client  # RightNow (sync)

    @overload
    def create(
        self,
        *,
        model: str,
        messages: list[dict[str, Any]],
        stream: bool = ...,
        **kwargs: Any,
    ) -> ChatCompletion: ...

    @overload
    def create(
        self,
        *,
        model: str,
        messages: list[dict[str, Any]],
        stream: bool = ...,
        **kwargs: Any,
    ) -> Stream: ...

    def create(
        self,
        *,
        model: str,
        messages: list[dict[str, Any]],
        stream: bool = False,
        **kwargs: Any,
    ) -> ChatCompletion | Stream:
        payload: dict[str, Any] = {"model": model, "messages": messages, "stream": stream, **kwargs}

        if stream:
            cm = self._client._http.stream(  # type: ignore[attr-defined]
                "POST",
                self._client._build_url("/v1/chat/completions"),  # type: ignore[attr-defined]
                json=payload,
                headers=self._client._headers,  # type: ignore[attr-defined]
                timeout=self._client.timeout,  # type: ignore[attr-defined]
            )
            raw = cm.__enter__()
            self._client._raise_for_status(raw)  # type: ignore[attr-defined]
            return Stream(raw, context_manager=cm)

        resp = self._client._http.post(  # type: ignore[attr-defined]
            self._client._build_url("/v1/chat/completions"),  # type: ignore[attr-defined]
            json=payload,
            headers=self._client._headers,  # type: ignore[attr-defined]
            timeout=self._client.timeout,  # type: ignore[attr-defined]
        )
        self._client._raise_for_status(resp)  # type: ignore[attr-defined]
        return ChatCompletion(**resp.json())


class AsyncCompletions:
    """Async chat completions: client.chat.completions.create(...)"""

    def __init__(self, client: object) -> None:
        self._client = client  # AsyncRightNow

    async def create(
        self,
        *,
        model: str,
        messages: list[dict[str, Any]],
        stream: bool = False,
        **kwargs: Any,
    ) -> ChatCompletion | AsyncStream:
        payload: dict[str, Any] = {"model": model, "messages": messages, "stream": stream, **kwargs}

        if stream:
            cm = self._client._http.stream(  # type: ignore[attr-defined]
                "POST",
                self._client._build_url("/v1/chat/completions"),  # type: ignore[attr-defined]
                json=payload,
                headers=self._client._headers,  # type: ignore[attr-defined]
                timeout=self._client.timeout,  # type: ignore[attr-defined]
            )
            response = await cm.__aenter__()
            self._client._raise_for_status(response)  # type: ignore[attr-defined]
            return AsyncStream(response, context_manager=cm)

        resp = await self._client._http.post(  # type: ignore[attr-defined]
            self._client._build_url("/v1/chat/completions"),  # type: ignore[attr-defined]
            json=payload,
            headers=self._client._headers,  # type: ignore[attr-defined]
            timeout=self._client.timeout,  # type: ignore[attr-defined]
        )
        self._client._raise_for_status(resp)  # type: ignore[attr-defined]
        return ChatCompletion(**resp.json())


class Chat:
    """Sync chat resource: client.chat.completions"""

    def __init__(self, client: object) -> None:
        self.completions = Completions(client)


class AsyncChat:
    """Async chat resource: client.chat.completions"""

    def __init__(self, client: object) -> None:
        self.completions = AsyncCompletions(client)



from __future__ import annotations


class RightNowError(Exception):
    """Base exception for the RightNow SDK."""

    def __init__(self, message: str, status_code: int | None = None, body: dict | None = None):
        super().__init__(message)
        self.status_code = status_code
        self.body = body


class AuthenticationError(RightNowError):
    """Raised when the API key is invalid or missing (401)."""


class PermissionDeniedError(RightNowError):
    """Raised when the API key lacks permission for the request (403)."""


class NotFoundError(RightNowError):
    """Raised when the requested resource does not exist (404)."""


class BadRequestError(RightNowError):
    """Raised when the request is malformed (400)."""


class RateLimitError(RightNowError):
    """Raised when rate-limited (429)."""


class InternalServerError(RightNowError):
    """Raised on upstream server errors (500+)."""


class APIConnectionError(RightNowError):
    """Raised when the SDK cannot connect to the API."""


class APITimeoutError(APIConnectionError):
    """Raised when a request times out."""


STATUS_MAP: dict[int, type[RightNowError]] = {
    400: BadRequestError,
    401: AuthenticationError,
    403: PermissionDeniedError,
    404: NotFoundError,
    429: RateLimitError,
}



from __future__ import annotations

from typing import Any

from .._types import DocumentResponse


class Documents:
    """Sync documents: client.documents.summarize/translate/extract/diacritize(...)"""

    def __init__(self, client: object) -> None:
        self._client = client

    def _post(self, path: str, payload: dict[str, Any]) -> DocumentResponse:
        resp = self._client._http.post(  # type: ignore[attr-defined]
            self._client._build_url(path),  # type: ignore[attr-defined]
            json=payload,
            headers=self._client._headers,  # type: ignore[attr-defined]
            timeout=self._client.timeout,  # type: ignore[attr-defined]
        )
        self._client._raise_for_status(resp)  # type: ignore[attr-defined]
        return DocumentResponse(**resp.json())

    def summarize(
        self,
        *,
        text: str,
        model: str = "falcon-h1-arabic-7b",
        language: str = "ar",
        max_length: int | None = None,
        **kwargs: Any,
    ) -> DocumentResponse:
        payload: dict[str, Any] = {
            "text": text,
            "model": model,
            "language": language,
            **kwargs,
        }
        if max_length is not None:
            payload["max_length"] = max_length
        return self._post("/v1/documents/summarize", payload)

    def translate(
        self,
        *,
        text: str,
        source_language: str = "ar",
        target_language: str = "en",
        model: str = "falcon-h1-arabic-7b",
        **kwargs: Any,
    ) -> DocumentResponse:
        payload: dict[str, Any] = {
            "text": text,
            "model": model,
            "source_language": source_language,
            "target_language": target_language,
            **kwargs,
        }
        return self._post("/v1/documents/translate", payload)

    def extract(
        self,
        *,
        text: str,
        entities: list[str],
        model: str = "falcon-h1-arabic-7b",
        **kwargs: Any,
    ) -> DocumentResponse:
        payload: dict[str, Any] = {
            "text": text,
            "model": model,
            "entities": entities,
            **kwargs,
        }
        return self._post("/v1/documents/extract", payload)

    def diacritize(
        self,
        *,
        text: str,
        model: str = "falcon-h1-arabic-7b",
        **kwargs: Any,
    ) -> DocumentResponse:
        payload: dict[str, Any] = {"text": text, "model": model, **kwargs}
        return self._post("/v1/documents/diacritize", payload)


class AsyncDocuments:
    """Async documents: client.documents.summarize/translate/extract/diacritize(...)"""

    def __init__(self, client: object) -> None:
        self._client = client

    async def _post(self, path: str, payload: dict[str, Any]) -> DocumentResponse:
        resp = await self._client._http.post(  # type: ignore[attr-defined]
            self._client._build_url(path),  # type: ignore[attr-defined]
            json=payload,
            headers=self._client._headers,  # type: ignore[attr-defined]
            timeout=self._client.timeout,  # type: ignore[attr-defined]
        )
        self._client._raise_for_status(resp)  # type: ignore[attr-defined]
        return DocumentResponse(**resp.json())

    async def summarize(
        self,
        *,
        text: str,
        model: str = "falcon-h1-arabic-7b",
        language: str = "ar",
        max_length: int | None = None,
        **kwargs: Any,
    ) -> DocumentResponse:
        payload: dict[str, Any] = {
            "text": text,
            "model": model,
            "language": language,
            **kwargs,
        }
        if max_length is not None:
            payload["max_length"] = max_length
        return await self._post("/v1/documents/summarize", payload)

    async def translate(
        self,
        *,
        text: str,
        source_language: str = "ar",
        target_language: str = "en",
        model: str = "falcon-h1-arabic-7b",
        **kwargs: Any,
    ) -> DocumentResponse:
        payload: dict[str, Any] = {
            "text": text,
            "model": model,
            "source_language": source_language,
            "target_language": target_language,
            **kwargs,
        }
        return await self._post("/v1/documents/translate", payload)

    async def extract(
        self,
        *,
        text: str,
        entities: list[str],
        model: str = "falcon-h1-arabic-7b",
        **kwargs: Any,
    ) -> DocumentResponse:
        payload: dict[str, Any] = {
            "text": text,
            "model": model,
            "entities": entities,
            **kwargs,
        }
        return await self._post("/v1/documents/extract", payload)

    async def diacritize(
        self,
        *,
        text: str,
        model: str = "falcon-h1-arabic-7b",
        **kwargs: Any,
    ) -> DocumentResponse:
        payload: dict[str, Any] = {"text": text, "model": model, **kwargs}
        return await self._post("/v1/documents/diacritize", payload)
